@@ -22,8 +22,8 @@
 
 #include <config.h>
 #include <string.h>
+
 #include <gtk/gtk.h>
-#include <gnome.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -33,7 +33,7 @@
 #include "window.h"
 
 
-#define GLADE_FILE "file_roller.glade"
+#define GLADE_FILE "file-roller.glade"
 #define TEMP_DOCS  "temp_docs"
 
 
