@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <fcntl.h>
 
 #include <glib.h>
 #include <libgnomevfs/gnome-vfs-types.h>
@@ -44,6 +45,64 @@ static void fr_command_cfile_finalize    (GObject             *object);
 static FRCommandClass *parent_class = NULL;
 
 
+static char *
+get_uncompressed_name_from_archive (FRCommand  *comm, 
+				    const char *e_filename)
+{
+	FRCommandCFile *comm_cfile = FR_COMMAND_CFILE (comm);
+	int             fd;
+	char            buffer[11];
+	char           *filename = NULL;
+	GString        *str = NULL;
+
+	if (comm_cfile->compress_prog != FR_COMPRESS_PROGRAM_GZIP)
+		return NULL;
+	
+	fd = open (e_filename, O_RDONLY);
+	if (fd < 0) 
+		return NULL;
+	
+	if (read (fd, &buffer, 11) < 11) {
+		close (fd);
+		return NULL;
+	}
+
+	/* Check whether the FLG.FNAME is set */
+	if (((unsigned char)(buffer[3]) & 0x08) != 0x08) {
+		close (fd);
+		return NULL;
+	}
+
+	/* Check whether the FLG.FEXTRA is set */
+	if (((unsigned char)(buffer[3]) & 0x04) == 0x04) {
+		close (fd);
+		return NULL;
+	}
+	
+	str = g_string_new ("");
+	/* Don't lose the first character */
+	g_string_append_c (str, buffer[10]);
+	while (read (fd, &buffer, 1) > 0) {
+		if (buffer[0] == '\0') {
+			close (fd);
+			filename = g_strdup (file_name_from_path (str->str));
+			g_string_free (str, TRUE);
+#ifdef DEBUG
+			g_message ("filename is: %s", filename);
+#endif
+			return filename;
+		}
+		
+		g_string_append_c (str, buffer[0]);
+	}
+	
+	close (fd);
+	g_string_free (str, TRUE);
+
+	return NULL;
+}
+
+
 static void
 list__process_line (char     *line, 
 		    gpointer  data)
@@ -56,13 +115,16 @@ list__process_line (char     *line,
 	fdata = file_data_new ();
 
 	fields = split_line (line, 2);
-	fdata->size = atol (fields[1]);
+	fdata->size = atoll (fields[1]);
 	g_strfreev (fields);
 
 	if (fdata->size == -1)
 		fdata->size = get_file_size (comm->filename);
 
-	filename = remove_extension_from_path (comm->filename);
+	filename = get_uncompressed_name_from_archive (comm, comm->filename);
+	if (filename == NULL) 
+		filename = remove_extension_from_path (comm->filename);
+
 	fdata->full_path = g_strconcat ("/", 
 					file_name_from_path (filename),
 					NULL);
@@ -150,7 +212,98 @@ fr_command_cfile_add (FRCommand     *comm,
 		      const char    *password,
 		      FRCompression  compression)
 {
-	/* never called */
+	FRCommandCFile *comm_cfile = FR_COMMAND_CFILE (comm);
+	const char     *filename;
+	char           *temp_dir;
+	char           *e_temp_dir;
+	char           *temp_file;
+
+	if ((file_list == NULL) || (file_list->data == NULL))
+		return;
+
+	/* create a temp dir. */
+
+	temp_dir = get_temp_work_dir ();
+	e_temp_dir = fr_command_escape (comm, temp_dir);
+
+	/* copy file to the temp dir */
+
+	filename = file_list->data;
+
+	temp_file = g_strconcat (e_temp_dir, "/", filename, NULL);
+
+	fr_process_begin_command (comm->process, "cp");
+	fr_process_set_working_dir (comm->process, base_dir);
+	fr_process_add_arg (comm->process, "-f");
+	fr_process_add_arg (comm->process, filename);
+	fr_process_add_arg (comm->process, temp_file);
+	fr_process_end_command (comm->process);
+
+	/**/
+
+	switch (comm_cfile->compress_prog) {
+	case FR_COMPRESS_PROGRAM_NONE:
+		break;
+		
+	case FR_COMPRESS_PROGRAM_GZIP:
+		fr_process_begin_command (comm->process, "gzip");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		break;
+
+	case FR_COMPRESS_PROGRAM_BZIP:
+		fr_process_begin_command (comm->process, "bzip");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		break;
+
+	case FR_COMPRESS_PROGRAM_BZIP2:
+		fr_process_begin_command (comm->process, "bzip2");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		break;
+
+	case FR_COMPRESS_PROGRAM_COMPRESS: 
+		fr_process_begin_command (comm->process, "compress");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		break;
+
+	case FR_COMPRESS_PROGRAM_LZOP: /* FIXME: untested. */
+		fr_process_begin_command (comm->process, "lzop");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "-fU");
+		fr_process_add_arg (comm->process, "--no-stdin");
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		break;
+	}
+
+      	/* copy compressed file to the dest dir */
+
+	fr_process_begin_command (comm->process, "cp");
+	fr_process_set_working_dir (comm->process, e_temp_dir);
+	fr_process_add_arg (comm->process, "-f");
+	fr_process_add_arg (comm->process, "*");
+	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_end_command (comm->process);
+
+	/* remove the temp dir */
+
+	fr_process_begin_command (comm->process, "rm");
+	fr_process_set_sticky (comm->process, TRUE);
+	fr_process_add_arg (comm->process, "-rf");
+	fr_process_add_arg (comm->process, e_temp_dir);
+	fr_process_end_command (comm->process);
+
+	g_free (temp_file);
+	g_free (e_temp_dir);
+	g_free (temp_dir);
 }
 
 
@@ -174,34 +327,32 @@ fr_command_cfile_extract (FRCommand  *comm,
 	FRCommandCFile *comm_cfile = FR_COMMAND_CFILE (comm);
 	char           *temp_dir;
 	char           *e_temp_dir;
-	char           *e_dest_dir;
+	char           *dest_file;
+	char           *e_dest_file;
 	char           *temp_file;
 	char           *uncompr_file;
-
-	temp_dir = get_temp_work_dir_name ();
-	e_temp_dir = shell_escape (temp_dir);
+	char           *compr_file;
 
 	/* create a temp dir. */
 
-	fr_process_begin_command (comm->process, "mkdir");
-	fr_process_add_arg (comm->process, e_temp_dir);
-	fr_process_end_command (comm->process);
+	temp_dir = get_temp_work_dir ();
+	e_temp_dir = fr_command_escape (comm, temp_dir);
 
-	/* copy file to the temp dir */
+	/* copy file to the temp dir, remove the already existing file first */
+
+	temp_file = g_strconcat (e_temp_dir,
+				 "/",
+				 file_name_from_path (comm->e_filename),
+				 NULL);
 
 	fr_process_begin_command (comm->process, "cp");
 	fr_process_add_arg (comm->process, "-f");
 	fr_process_add_arg (comm->process, comm->e_filename);
-	fr_process_add_arg (comm->process, e_temp_dir);
+	fr_process_add_arg (comm->process, temp_file);
 	fr_process_end_command (comm->process);
 
 	/* uncompress the file */
 
-	temp_file = g_strconcat (e_temp_dir, 
-				 "/", 
-				 file_name_from_path (comm->e_filename), 
-				 NULL);
-
 	switch (comm_cfile->compress_prog) {
 	case FR_COMPRESS_PROGRAM_NONE:
 		break;
@@ -210,6 +361,7 @@ fr_command_cfile_extract (FRCommand  *comm,
 		fr_process_begin_command (comm->process, "gzip");
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, "-d");
+		fr_process_add_arg (comm->process, "-n");
 		fr_process_add_arg (comm->process, temp_file);
 		fr_process_end_command (comm->process);
 		break;
@@ -251,12 +403,24 @@ fr_command_cfile_extract (FRCommand  *comm,
 	/* copy uncompress file to the dest dir */
 
 	uncompr_file = remove_extension_from_path (temp_file);
-	e_dest_dir = shell_escape (dest_dir);
+
+	compr_file = get_uncompressed_name_from_archive (comm, comm->filename);
+	if (compr_file == NULL) {
+		g_print ("[0]\n");
+		compr_file = remove_extension_from_path (file_name_from_path (comm->e_filename));
+	}
+	dest_file = g_strconcat (dest_dir,
+				 "/",
+				 compr_file,
+				 NULL);
+	e_dest_file = fr_command_escape (comm, dest_file);
+	g_free (compr_file);
+	g_free (dest_file);
 
 	fr_process_begin_command (comm->process, "cp");
 	fr_process_add_arg (comm->process, "-f");
 	fr_process_add_arg (comm->process, uncompr_file);
-	fr_process_add_arg (comm->process, e_dest_dir);
+	fr_process_add_arg (comm->process, e_dest_file);
 	fr_process_end_command (comm->process);
 
 	/* remove the temp dir */
@@ -267,7 +431,7 @@ fr_command_cfile_extract (FRCommand  *comm,
 	fr_process_add_arg (comm->process, e_temp_dir);
 	fr_process_end_command (comm->process);
 
-	g_free (e_dest_dir);
+	g_free (e_dest_file);
 	g_free (uncompr_file);
 	g_free (temp_file);
 	g_free (e_temp_dir);
@@ -357,6 +521,32 @@ fr_command_cfile_new (FRProcess         *process,
 {
 	FRCommand *comm;
 
+	if ((prog == FR_COMPRESS_PROGRAM_GZIP) &&
+	    (!is_program_in_path("gzip"))) {
+		return NULL;
+	}
+
+	if ((prog == FR_COMPRESS_PROGRAM_BZIP) &&
+	    (!is_program_in_path("bzip"))) { 
+		return NULL;
+	}
+
+	if ((prog == FR_COMPRESS_PROGRAM_BZIP2) &&
+	    (!is_program_in_path("bzip2"))) {
+		return NULL;
+	}
+
+	if ((prog == FR_COMPRESS_PROGRAM_COMPRESS) && 
+	    ((!is_program_in_path("compress")) || 
+	     (!is_program_in_path("uncompress")))) {
+		return NULL;
+	}
+
+	if ((prog == FR_COMPRESS_PROGRAM_LZOP) &&
+	    (!is_program_in_path("lzop"))) {
+		return NULL;
+	}
+
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_CFILE, NULL));
 	fr_command_construct (comm, process, filename);
 	FR_COMMAND_CFILE (comm)->compress_prog = prog;
