@@ -22,15 +22,17 @@
 
 #include <config.h>
 
+#include <sys/types.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
-#include <gnome.h>
 
 #include "file-data.h"
 #include "file-utils.h"
@@ -72,7 +74,7 @@ mktime_from_string (char *date_s,
 
 	fields = g_strsplit (time_s, ":", 3);
 	if (fields[0] != NULL) {
-		tm.tm_hour = atoi (fields[0]) - 1;
+		tm.tm_hour = atoi (fields[0]);
 		if (fields[1] != NULL) {
 			tm.tm_min  = atoi (fields[1]);
 			if (fields[2] != NULL)
@@ -132,7 +134,7 @@ process_line (char     *line,
 	date_idx = file_list__get_index_from_pattern (line, "%n%n%n%n-%n%n-%n%n %n%n:%n%n");
 
 	field_size = file_list__get_prev_field (line, date_idx, 1);
-	fdata->size = atol (field_size);
+	fdata->size = atoll (field_size);
 	g_free (field_size);
 
 	field_date = file_list__get_next_field (line, date_idx, 1);
@@ -212,26 +214,21 @@ add_compress_arg (FRCommand *comm)
 static void
 begin_tar_command (FRCommand *comm)
 {
-	char       *path = NULL;
-	const char *prev_path = NULL;
-	char       *temp = NULL;
+	char *command = NULL;
 
 	/* In solaris gtar is present under /usr/sfw/bin */
 
-	prev_path = g_getenv ("PATH");
-	temp = g_strdup_printf ("PATH=%s:%s", prev_path, "/usr/sfw/bin");
-	putenv (temp);
-
-	path = g_find_program_in_path ("gtar");
-	if (path != NULL)
-		fr_process_begin_command (comm->process, path);
+	command = g_find_program_in_path ("gtar");
+#if defined (__SVR4) && defined (__sun)
+	if (g_file_test ("/usr/sfw/bin/gtar", G_FILE_TEST_IS_EXECUTABLE)) {
+		command = g_strdup ("/usr/sfw/bin/gtar");
+	}
+#endif
+	if (command != NULL)
+		fr_process_begin_command (comm->process, command);
 	else
 		fr_process_begin_command (comm->process, "tar");
-
-	temp = g_strdup_printf ("PATH=%s", prev_path);
-	putenv (temp);
-
-	g_free (path);
+	g_free (command);
 }
 
 
@@ -862,6 +859,39 @@ fr_command_tar_new (FRProcess         *process,
 {
 	FRCommand *comm;
 
+	/* In solaris gtar is present under /usr/sfw/bin */
+
+	if ((!is_program_in_path("/usr/sfw/bin/gtar")) &&
+	    (!is_program_in_path("tar"))) {
+		return NULL;
+	}
+
+	if ((prog == FR_COMPRESS_PROGRAM_GZIP) &&
+            (!is_program_in_path("gzip"))) {
+                return NULL;
+        }
+
+        if ((prog == FR_COMPRESS_PROGRAM_BZIP) &&
+            (!is_program_in_path("bzip"))) {
+                return NULL;
+        }
+
+        if ((prog == FR_COMPRESS_PROGRAM_BZIP2) &&
+            (!is_program_in_path("bzip2"))) {
+                return NULL;
+        }
+
+        if ((prog == FR_COMPRESS_PROGRAM_COMPRESS) &&
+            ((!is_program_in_path("compress")) ||
+             (!is_program_in_path("uncompress")))) {
+                return NULL;
+        }
+
+        if ((prog == FR_COMPRESS_PROGRAM_LZOP) &&
+            (!is_program_in_path("lzop"))) {
+                return NULL;
+        }
+
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_TAR, NULL));
 	fr_command_construct (comm, process, filename);
 	FR_COMMAND_TAR (comm)->compress_prog = prog;
