@@ -387,7 +387,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	if (gtk_stock_lookup (stock_id, &item))
 		title = item.label;
 	else
-		title = _("File Roller");
+		title = _("Archive Manager");
 
 	dialog = gtk_dialog_new_with_buttons (title, parent, flags, 
 					      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
@@ -496,6 +496,37 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 }
 
 
+void
+_gtk_error_dialog_run (GtkWindow        *parent,
+		       const gchar      *main_message,
+		       const gchar      *format,
+		       ...)
+{
+	GtkWidget *d;
+	char      *message;
+	va_list    args;
+
+	va_start (args, format);
+	message = g_strdup_vprintf (format, args);
+	va_end (args);
+
+	d =  _gtk_message_dialog_new (parent,
+				      GTK_DIALOG_MODAL,
+				      GTK_STOCK_DIALOG_ERROR,
+				      main_message,
+				      message,
+				      GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
+				      NULL);
+	g_free (message);
+
+	g_signal_connect (G_OBJECT (d), "response",
+			  G_CALLBACK (gtk_widget_destroy),
+			  NULL);
+
+	gtk_widget_show (d);
+}
+
+
 void
 _gtk_entry_set_locale_text (GtkEntry   *entry,
 			    const char *text)
@@ -503,7 +534,10 @@ _gtk_entry_set_locale_text (GtkEntry   *entry,
 	char *utf8_text;
 
 	utf8_text = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
-	gtk_entry_set_text (entry, utf8_text);
+	if (utf8_text != NULL)
+		gtk_entry_set_text (entry, utf8_text);
+	else
+		gtk_entry_set_text (entry, "");
 	g_free (utf8_text);
 }
 
@@ -531,8 +565,11 @@ _gtk_label_set_locale_text (GtkLabel   *label,
 	char *utf8_text;
 
 	utf8_text = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
-	gtk_label_set_text (label, utf8_text);
-	g_free (utf8_text);
+	if (utf8_text != NULL) {
+		gtk_label_set_text (label, utf8_text);
+		g_free (utf8_text);
+	} else
+		gtk_label_set_text (label, "");
 }
 
 
@@ -559,8 +596,11 @@ _gtk_entry_set_filename_text (GtkEntry   *entry,
 	char *utf8_text;
 
 	utf8_text = g_filename_to_utf8 (text, -1, NULL, NULL, NULL);
-	gtk_entry_set_text (entry, utf8_text);
-	g_free (utf8_text);
+	if (utf8_text != NULL) {
+		gtk_entry_set_text (entry, utf8_text);
+		g_free (utf8_text);
+	} else
+		gtk_entry_set_text (entry, "");
 }
 
 
@@ -586,7 +626,7 @@ _gtk_label_set_filename_text (GtkLabel   *label,
 {
 	char *utf8_text;
 
-	utf8_text = g_filename_to_utf8 (text, -1, NULL, NULL, NULL);
+	utf8_text = g_filename_display_name (text);
 	gtk_label_set_text (label, utf8_text);
 	g_free (utf8_text);
 }
