@@ -86,6 +86,8 @@ typedef struct {
 	GtkWidget *      fwd_button;
 	GtkCellRenderer *name_renderer;
 
+	gint             current_view_length;
+
 	GtkTooltips     *tooltips;
 	guint            help_message_cid;
 	guint            list_info_cid;
@@ -148,7 +150,7 @@ typedef struct {
 
 	GtkActionGroup  *actions;
 
-	EggRecentViewUIManager *recent_view;
+	EggRecentViewGtk *recent_view_menu;
 	EggRecentViewGtk *recent_view_toolbar;
 	EggRecentModel   *recent_model;
 	GtkWidget        *file_popup_menu;
